package Handel::L10N::en_us;
use strict;
use warnings;
use utf8;
use vars qw(%Lexicon);

BEGIN {
    use base 'Handel::L10N';
};

%Lexicon = (
    Language => 'English',
);

1;
__END__

=head1 NAME

Handel::L10N::en_us - US English Language Pack for Handel

=head1 VERSION

    $Id: en_us.pm 33 2004-12-31 02:12:40Z claco $

=head1 AUTHOR

    Christopher H. Laco
    CPAN ID: CLACO
    cpan@chrislaco.com
    http://today.icantfocus.com/blog/